/*  COMCLK.C
    Peruen a asovn 1/100 s od vyslae sriov linky.
    (C) Ivan Doleal
    20. 12. 1994
*/

#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <conio.h>

/* COM - Posuv adres vybranch registr vi bzov adrese   */
#define TXR    0     /* Transmit register (WRITE)            */
#define DLL    0     /* Divisor Latch Low  - registr dlie  */
#define DLH    1     /* Divisor Latch High - registr dlie  */
#define IER    1     /* Interrupt Enable Register            */
#define IIR    2     /* Interrupt Identification Register    */
#define LCR    3     /* Line Control Register                */
#define MCR    4     /* Modem Control Register               */

/* COM: vybran bity */
#define INTR_EN      0x08    /* MCR: Povolen signlu peruen od COM portu */
#define BREAK_EN     0x40    /* LCR: Bit petren - na vstupu trvale log.0 */
#define TR_INT       0x02    /* IER: Kd volby zdroje peruen - vysla */
#define TR_ID        0x02    /* IIR: Kd identifikace zdroje peruen - vysla */

/* PIC 8259 - registry a kdy adie peruen */
#define IMR          0x21    /* Interrupt Mask Register */
#define ICR          0x20    /* Interrupt Control Register */
#define EOI          0x20    /* End Of Interrupt */
#define IRQ(n)       (1<<n)  /* maska peruen */

int
   port = 2,           /* volba portu: COM2 */
   portbase = 0x2F8,   /* bzov adresa; COM1: 3F8h, COM3: 3E8h, COM4: 2E8h */
   run = 1,            /* pznak zapnut bhu asu */
   int_no = 0x0B,      /* . vektoru peruen; COM1: 0Ch, COM3: 0Ch, COM4: 0Bh */
   irq_mask = IRQ(3),  /* maska peruen pro IMR; COM1: 4, COM3: 4, COM4: 3 */
   oldMCR, oldLCR, oldIMR;         /* pomocn promnn */
void interrupt (*oldcomvect)();
struct TIME  {  char s100;  char sec;  char min;  char hrs; };
volatile struct TIME  t = { 0, 0, 0, 0 };      /* inicializace asovho daje */
volatile unsigned long s100clk = 0;            /* ta peruen po 1/100 s */

/***************************/
void interrupt com_intr()                /* Rutina obsluhy peruen od COM */
{
if (run)                                 /* test zapnut bhu asu */
   if (++t.s100 == 100)  {
      t.s100 = 0;
      if (++t.sec == 60)   {
         t.sec = 0;
         if (++t.min == 60)    {
            t.min = 0;
            if (++t.hrs == 24)
               t.hrs = 0;      }  }  }   /* inkrementace a oeten peteen */
s100clk++;                               /* inkrementace tae peruen */
outportb( portbase+TXR, 0 );             /* vysln znaku pro dal peruen */
outportb(ICR,EOI);                       /* potvrzen peruen v adii */
}

/***************************/
void setbaudrate(unsigned divisor)       /* Funkce nastaven penosov rychlosti */
{
char  c;
if (divisor == 0)                        /* oeten event. dlen nulou */
   divisor = 1;
disable();
c = inportb(portbase+LCR);
outportb(portbase+LCR, (c | 0x80));           /* Nastaven DLAB - volba DLL, DLH */
outportb(portbase+DLL, (divisor & 0x00FF));
outportb(portbase+DLH, ((divisor >> 8) & 0x00FF));
outportb(portbase+LCR, c);                    /* Nulovn DLAB - volba TXR, IER */
enable();
return;
}

/***************************/
void com_intr_en(void)                   /* Funkce instalace obsluhy peruen */
{
oldcomvect = getvect(int_no);
setvect(int_no, com_intr);               /* nastaven uivatelsk obsluhy peru. */
disable();
oldLCR = inportb(portbase+LCR);
outportb( portbase+LCR, 0x0F | BREAK_EN );   /* 8 datovch, paritn, 2 stop bity */
oldMCR = inportb(portbase+MCR);
outportb( portbase+MCR, oldMCR | INTR_EN );  /* povolen vstupu signlu peru. */
inportb(portbase+IIR);                       /* reset IIR jeho tenm */
outportb( portbase+IER, TR_INT );            /* volba zdroje peruen - vysla */
oldIMR = inportb(IMR);
outportb( IMR, oldIMR & ~irq_mask );         /* odmaskovn peru. od COM v PIC */
enable();
outportb( portbase+TXR, 0 );                 /* vysln znaku pro 1. peruen */
}

/***************************/
void com_intr_dis(void)                  /* Funkce deinstalace obsluhy peruen */
{
disable();
outportb( IMR, oldIMR );
outportb( portbase+IER, 0 );
outportb( portbase+MCR, oldMCR );
outportb( portbase+LCR, oldLCR );        /* vude obnova pvodnch nastaven */
enable();
setvect(int_no, oldcomvect);             /* obnova pvodnho vektoru peruen */
}

/***************************/
int main(void)
{
unsigned long s100clk0 = 0;
puts("\n- COMCLK - Mereni casu po 1/100 s vyuzitim preruseni COM - (C) I. Dolezal 1994 -"
       "  Ukonceni stiskem <ENTER>.\n");
setbaudrate(96);                         /* nastaven dlitele pro 1/100 s */
_setcursortype(_NOCURSOR);               /* .. aby pi pebzch neblikal kurzor */
com_intr_en();                           /* instalace obsluhy peruen */
do if ( s100clk >= s100clk0 )            /* zobrazen po kadm n-tm peruen */
      {
      s100clk0 += 7;                     /* volba prodlevy do dalho zobrazen */
      printf("Cas od startu -> %hu:%02hu:%02hu.%02hu\r",
              t.hrs, t.min, t.sec, t.s100 );
      }                                  /* zobrazen asu */
while ( ! (kbhit() && getch()=='\r') );  /* ek na stisk ENTER */
com_intr_dis();                          /* odinstalace obsluhy peruen */
_setcursortype(_NORMALCURSOR);           /* obnoven kurzoru */
return (0);
}
