{************************************************}
{                                                }
{   Program Latin2 Kamenicky                     }
{   Robert Plzak                                 }
{   Turbo Pascal 6.0                             }
{   Prevod zdrojovych souboru mezi kody Latin2,  }
{   Kamenickych a ASCII                          }
{                                                }
{************************************************}

program L2K;

{$I-}

uses Dos, Crt;

type
  Buff = array [1..2048] of Char;        { kdovac bafr }
  TCharFnc = function(C: Char): Char;    { ukazatel na pevodn fuknci }

const
{ Pevodn tabulka, jednodue uloen v etzci }
  CSStr: String[114]  = #130#131#135#136#145#141#150#152#159 +
                        #160#161#162#147#163#164#168#140#169 +
  { Kamenickch }       #170#144#133#128#137#146#138#166#157 +
                        #134#143#139#149#167#151#165#155#156#158#171 +

  { Ascii }             'edcezluytaioounslrrEDCEZLUYTAIOOUNSLRR' +

                        #130#212#159#216#167#146#133#236#156 +
                        #160#161#162#147#163#229#231#150#253 +
  { Latin2 }            #234#144#210#172#183#166#145#222#237 +
                        #155#181#214#224#226#233#213#230#149#252#232;
var
  FromF, ToF: File;             { vstupn a vstupn soubor }
  FromS, ToS: PathStr;
  Par: String;                  { parametr pevodu }
  NumRead, NumWritten: Word;    { nateno a zapsno }
  Buffer: Buff;                 { naten sek }
  Ch: Char;
  Prevod: TCharFnc;             { aktuln pevodn funkce }

function UpStr(S: String): String;
{ pevod na velk psmena }
var
  I: Byte;
begin
  For I := 1 to Byte(S[0]) do
      S[I] := UpCase(S[I]);
  UpStr := S;
end;

function FileExist(F : String) : Boolean;
{ kontrola existence vstupnho souboru }
var
  SR : SearchRec;
begin
  FindFirst(F, AnyFile, SR);
  FileExist := DosError = 0;
end;

function LatinKam(C: Char): Char; assembler;
{ pokud nalezne znak C v Latin2 sti etzce
  CSStr, pevede ho na znak v kdu Kamenickch }
asm
      MOV  AL,C
      LEA  SI,CSStr[77]
      MOV  DI,SI
      MOV  BL,76
      CMP  AL,130
      JB   @@KN
      CMP  AL,253
      JA   @@KN
@@PO: INC  BL
      CMP  BL,115
      JE   @@KN
      CMP  AL,DS:[SI]
      JE   @@LA
      INC  SI
      JMP  @@PO
@@LA: SUB  SI,76
      MOV  AL,DS:[SI]
@@KN:
end;

function KamLatin(C: Char): Char; assembler;
{ pokud nalezne znak C v Kamenickch sti etzce
  CSStr, pevede ho na znak v kdu Latin2 }
asm
      MOV  AL,C
      LEA  SI,CSStr[1]
      MOV  DI,SI
      MOV  BL,0
      CMP  AL,128
      JB   @@KN
      CMP  AL,171
      JA   @@KN
@@PO: INC  BL
      CMP  BL,39
      JE   @@KN
      CMP  AL,DS:[SI]
      JE   @@AS
      INC  SI
      JMP  @@PO
@@AS: ADD  SI,76
      MOV  AL,DS:[SI]
@@KN:
end;

function LatinAscii(C: Char): Char; assembler;
{ pokud nalezne znak C v Latin2 sti etzce
  CSStr, pevede ho na znak v kdu ASCII }
asm
      MOV  AL,C
      LEA  SI,CSStr[77]
      MOV  DI,SI
      MOV  BL,76
      CMP  AL,130
      JB   @@KN
      CMP  AL,253
      JA   @@KN
@@PO: INC  BL
      CMP  BL,115
      JE   @@KN
      CMP  AL,DS:[SI]
      JE   @@LA
      INC  SI
      JMP  @@PO
@@LA: SUB  SI,38
      MOV  AL,DS:[SI]
@@KN:
end;

function KamAscii(C: Char): Char; assembler;
{ pokud nalezne znak C v Kamenickch sti etzce
  CSStr, pevede ho na znak v kdu ASCII }
asm
      MOV  AL,C
      LEA  SI,CSStr[1]
      MOV  DI,SI
      MOV  BL,0
      CMP  AL,128
      JB   @@KN
      CMP  AL,171
      JA   @@KN
@@PO: INC  BL
      CMP  BL,39
      JE   @@KN
      CMP  AL,DS:[SI]
      JE   @@AS
      INC  SI
      JMP  @@PO
@@AS: ADD  SI,38
      MOV  AL,DS:[SI]
@@KN:
end;

procedure Filtruj;
{ pomoc funkce, jej ukazatel je uloen
  v promnn, pevod pevede naten buffer}
var
  I: Word;
begin
  For I := 1 to SizeOf(Buffer) do
     Buffer[I] := Prevod(Buffer[I]);
end;

function JakyMod(Par: String): String;
{ vype hlen podle parametru }
begin
   If Par = 'LK' then
      JakyMod := 'Latin2 na b.Kamenickch'
   else
   If Par = 'KL' then
      JakyMod := 'b.Kamenickch na Latin2'
   else
   If Par = 'LA' then
      JakyMod := 'Latin2 na Ascii'
   else
   If Par = 'KA' then
      JakyMod := 'b.Kamenickch na Ascii';
end;

procedure ErrorHalt;
{ vype sprvnou syntaxi a ukon program }
begin
  Writeln;
  Writeln('Syntaxe:  L2K <Vstupn_Soubor> <Vstupn_Soubor> <Parametr>');
  Writeln('Parametr: LK - ', JakyMod('LK'));
  Writeln('          KL - ', JakyMod('KL'));
  Writeln('          LA - ', JakyMod('LA'));
  Writeln('          KA - ', JakyMod('KA'));
  Writeln;
  Halt(1);
end;

procedure GetParamOfDos;
{ kontrola zadanch parametr
  a uren pevodn funkce }
begin
  If ParamCount >= 3 then
  begin
    FromS := FExpand(ParamStr(1));
    ToS := FExpand(ParamStr(2));
    Par := UpStr(ParamStr(3));
    If Par = 'LK' then
       Prevod := LatinKam
    else
    If Par = 'KL' then
       Prevod := KamLatin
    else
    If Par = 'LA' then
       Prevod := LatinAscii
    else
    If Par = 'KA' then
       Prevod := KamAscii
    else ErrorHalt;
  end
  else ErrorHalt;
@LH 6
end;
@LH 3

{ vlastn cyklus programu }
begin
  Writeln(^J^M, 'Program pro pevod mezi Latin2, b.Kamenickch a Ascii.');
  GetParamOfDos;
  If FromS = ToS then ErrorHalt;
  Assign(FromF, FromS);
  Reset(FromF, 1);
  If IoResult <> 0 then ErrorHalt;
  If FileExist(ToS) then
  begin
     Write('Pepsat ', ToS, ' A/N ?');
     Ch := ReadKey;
     If UpCase(Ch) <> 'A' then ErrorHalt;
  end;
  Assign(ToF, ToS);
  Rewrite(ToF, 1);
  If IoResult <> 0 then ErrorHalt;
  Writeln;
  Writeln('Pevod z ', FromS, ^J^M, '      do ',
           ToS, ^J^M, '    Md: ', JakyMod(Par));
  Repeat
     BlockRead(FromF, Buffer, SizeOf(Buffer), NumRead);
     Filtruj;
     BlockWrite(ToF, Buffer, NumRead, NumWritten);
  Until (NumRead = 0) or (NumWritten <> NumRead);
  Close(ToF);
  Close(FromF);
end.